@@ -133,17 +133,18 @@ class NinjaBackend(backends.Backend):
         self.all_outputs = {}
         self.valgrind = environment.find_valgrind()
 
-    def detect_vs_dep_prefix(self, outfile, tempfilename):
+    def detect_vs_dep_prefix(self, tempfilename):
         '''VS writes its dependency in a locale dependent format.
         Detect the search prefix to use.'''
         # Of course there is another program called 'cl' on
         # some platforms. Let's just require that on Windows
         # cl points to msvc.
         if not mesonlib.is_windows() or shutil.which('cl') is None:
-            return outfile
-        outfile.close()
-        open(os.path.join(self.environment.get_scratch_dir(), 'incdetect.c'),
-              'w').write('''#include<stdio.h>
+            return open(tempfilename, 'a')
+        filename = os.path.join(self.environment.get_scratch_dir(),
+                                'incdetect.c')
+        with open(filename, 'w') as f:
+            f.write('''#include<stdio.h>
 int dummy;
 ''')
 
@@ -157,9 +158,8 @@ int dummy;
         for line in stdo.split(b'\r\n'):
             if line.endswith(b'stdio.h'):
                 matchstr = b':'.join(line.split(b':')[0:2]) + b':'
-                binfile = open(tempfilename, 'ab')
-                binfile.write(b'msvc_deps_prefix = ' + matchstr + b'\r\n')
-                binfile.close()
+                with open(tempfilename, 'ab') as binfile:
+                    binfile.write(b'msvc_deps_prefix = ' + matchstr + b'\r\n')
                 return open(tempfilename, 'a')
         raise MesonException('Could not determine vs dep dependency prefix string.')
 
@@ -167,30 +167,31 @@ int dummy;
         self.interpreter = interp
         outfilename = os.path.join(self.environment.get_build_dir(), self.ninja_filename)
         tempfilename = outfilename + '~'
-        outfile = open(tempfilename, 'w')
-        outfile.write('# This is the build file for project "%s"\n' % self.build.get_project())
-        outfile.write('# It is autogenerated by the Meson build system.\n')
-        outfile.write('# Do not edit by hand.\n\n')
-        outfile.write('ninja_required_version = 1.5.1\n\n')
-        outfile = self.detect_vs_dep_prefix(outfile, tempfilename)
-        self.generate_rules(outfile)
-        self.generate_phony(outfile)
-        outfile.write('# Build rules for targets\n\n')
-        [self.generate_target(t, outfile) for t in self.build.get_targets().values()]
-        outfile.write('# Test rules\n\n')
-        self.generate_tests(outfile)
-        outfile.write('# Install rules\n\n')
-        self.generate_install(outfile)
-        if 'b_coverage' in self.environment.coredata.base_options and \
-            self.environment.coredata.base_options['b_coverage'].value:
-            outfile.write('# Coverage rules\n\n')
-            self.generate_coverage_rules(outfile)
-        outfile.write('# Suffix\n\n')
-        self.generate_utils(outfile)
-        self.generate_ending(outfile)
+        with open(tempfilename, 'w') as outfile:
+            outfile.write('# This is the build file for project "%s"\n' %
+                          self.build.get_project())
+            outfile.write('# It is autogenerated by the Meson build system.\n')
+            outfile.write('# Do not edit by hand.\n\n')
+            outfile.write('ninja_required_version = 1.5.1\n\n')
+        with self.detect_vs_dep_prefix(tempfilename) as outfile:
+            self.generate_rules(outfile)
+            self.generate_phony(outfile)
+            outfile.write('# Build rules for targets\n\n')
+            for t in self.build.get_targets().values():
+                self.generate_target(t, outfile)
+            outfile.write('# Test rules\n\n')
+            self.generate_tests(outfile)
+            outfile.write('# Install rules\n\n')
+            self.generate_install(outfile)
+            if 'b_coverage' in self.environment.coredata.base_options and \
+                    self.environment.coredata.base_options['b_coverage'].value:
+                outfile.write('# Coverage rules\n\n')
+                self.generate_coverage_rules(outfile)
+            outfile.write('# Suffix\n\n')
+            self.generate_utils(outfile)
+            self.generate_ending(outfile)
         # Only ovewrite the old build file after the new one has been
         # fully created.
-        outfile.close()
         os.replace(tempfilename, outfilename)
         self.generate_compdb()
 
@@ -202,7 +203,8 @@ int dummy;
             jsondb = subprocess.check_output([ninja_exe, '-t', 'compdb', 'c_COMPILER', 'cpp_COMPILER'], cwd=builddir)
         except Exception:
                 raise MesonException('Could not create compilation database.')
-        open(os.path.join(builddir, 'compile_commands.json'), 'wb').write(jsondb)
+        with open(os.path.join(builddir, 'compile_commands.json'), 'wb') as f:
+            f.write(jsondb)
 
     # Get all generated headers. Any source file might need them so
     # we need to add an order dependency to them.
@@ -366,7 +368,11 @@ int dummy;
         desc = 'Generating {0} with a {1} command.'
         if target.build_always:
             deps.append('PHONY')
-        elem = NinjaBuildElement(self.all_outputs, ofilenames, 'CUSTOM_COMMAND', srcs)
+        if target.depfile is None:
+            rulename = 'CUSTOM_COMMAND'
+        else:
+            rulename = 'CUSTOM_COMMAND_DEP'
+        elem = NinjaBuildElement(self.all_outputs, ofilenames, rulename, srcs)
         for i in target.depend_files:
             if isinstance(i, mesonlib.File):
                 deps.append(i.rel_to_builddir(self.build_to_src))
@@ -379,21 +385,30 @@ int dummy;
                 tmp = [tmp]
             for fname in tmp:
                 elem.add_dep(os.path.join(self.get_target_dir(d), fname))
+        # If the target requires capturing stdout, then use the serialized
+        # executable wrapper to capture that output and save it to a file.
+        #
         # Windows doesn't have -rpath, so for EXEs that need DLLs built within
         # the project, we need to set PATH so the DLLs are found. We use
         # a serialized executable wrapper for that and check if the
         # CustomTarget command needs extra paths first.
-        if mesonlib.is_windows() and \
-           self.determine_windows_extra_paths(target.command[0]):
+        if target.capture or (mesonlib.is_windows() and
+                self.determine_windows_extra_paths(target.command[0])):
             exe_data = self.serialise_executable(target.command[0], cmd[1:],
                 # All targets are built from the build dir
-                self.environment.get_build_dir())
+                self.environment.get_build_dir(),
+                capture=ofilenames[0] if target.capture else None)
             cmd = [sys.executable, self.environment.get_build_command(),
                    '--internal', 'exe', exe_data]
             cmd_type = 'meson_exe.py custom'
         else:
             cmd_type = 'custom'
 
+        if target.depfile is not None:
+            rel_dfile = os.path.join(self.get_target_private_dir(target), target.depfile)
+            abs_pdir = os.path.join(self.environment.get_build_dir(), self.get_target_private_dir(target))
+            os.makedirs(abs_pdir, exist_ok=True)
+            elem.add_item('DEPFILE', rel_dfile)
         elem.add_item('COMMAND', cmd)
         elem.add_item('description',  desc.format(target.name, cmd_type))
         elem.write(outfile)
@@ -492,8 +507,8 @@ int dummy;
         self.generate_subdir_install(d)
         elem.write(outfile)
 
-        ofile = open(install_data_file, 'wb')
-        pickle.dump(d, ofile)
+        with open(install_data_file, 'wb') as ofile:
+            pickle.dump(d, ofile)
 
     def generate_target_install(self, d):
         should_strip = self.environment.coredata.get_builtin_option('strip')
@@ -636,7 +651,6 @@ int dummy;
             velem.write(outfile)
 
         # And then benchmarks.
-        benchmark_script = os.path.join(script_root, 'meson_benchmark.py')
         cmd = [sys.executable, self.environment.get_build_command(), '--internal', 'benchmark', benchmark_data]
         elem = NinjaBuildElement(self.all_outputs, 'benchmark', 'CUSTOM_COMMAND', ['all', 'PHONY'])
         elem.add_item('COMMAND', cmd)
@@ -657,6 +671,14 @@ int dummy;
         outfile.write(' command = $COMMAND\n')
         outfile.write(' description = $DESC\n')
         outfile.write(' restat = 1\n\n')
+        # Ninja errors out if you have deps = gcc but no depfile, so we must
+        # have two rules for custom commands.
+        outfile.write('rule CUSTOM_COMMAND_DEP\n')
+        outfile.write(' command = $COMMAND\n')
+        outfile.write(' description = $DESC\n')
+        outfile.write(' deps = gcc\n')
+        outfile.write(' depfile = $DEPFILE\n')
+        outfile.write(' restat = 1\n\n')
         outfile.write('rule REGENERATE_BUILD\n')
         c = (quote_char + ninja_quote(sys.executable) + quote_char,
              quote_char + ninja_quote(self.environment.get_build_command())  + quote_char,
@@ -1351,8 +1373,16 @@ rule FORTRAN_DEP_HACK
             infilename = os.path.join(self.build_to_src, curfile)
             outfiles = genlist.get_outputs_for(curfile)
             outfiles = [os.path.join(self.get_target_private_dir(target), of) for of in outfiles]
+            if generator.depfile is None:
+                rulename = 'CUSTOM_COMMAND'
+                args = base_args
+            else:
+                rulename = 'CUSTOM_COMMAND_DEP'
+                depfilename = generator.get_dep_outname(infilename)
+                depfile = os.path.join(self.get_target_private_dir(target), depfilename)
+                args = [x.replace('@DEPFILE@', depfile)  for x in base_args]
             args = [x.replace("@INPUT@", infilename).replace('@OUTPUT@', sole_output)\
-                    for x in base_args]
+                    for x in args]
             args = self.replace_outputs(args, self.get_target_private_dir(target), outfilelist)
             # We have consumed output files, so drop them from the list of remaining outputs.
             if sole_output == '':
@@ -1361,7 +1391,9 @@ rule FORTRAN_DEP_HACK
             args = [x.replace("@SOURCE_DIR@", self.build_to_src).replace("@BUILD_DIR@", relout)
                     for x in args]
             cmdlist = exe_arr + self.replace_extra_args(args, genlist)
-            elem = NinjaBuildElement(self.all_outputs, outfiles, 'CUSTOM_COMMAND', infilename)
+            elem = NinjaBuildElement(self.all_outputs, outfiles, rulename, infilename)
+            if generator.depfile is not None:
+                elem.add_item('DEPFILE', depfile)
             if len(extra_dependencies) > 0:
                 elem.add_dep(extra_dependencies)
             elem.add_item('DESC', 'Generating $out')
@@ -1386,16 +1418,22 @@ rule FORTRAN_DEP_HACK
             # but those are really rare. I hope.
             if not compiler.can_compile(s):
                 continue
-            for line in open(os.path.join(self.environment.get_source_dir(), s.subdir, s.fname)):
-                modmatch = modre.match(line)
-                if modmatch is not None:
-                    modname = modmatch.group(1)
-                    if modname.lower() == 'procedure': # MODULE PROCEDURE construct
-                        continue
-                    if modname in module_files:
-                        raise InvalidArguments('Namespace collision: module %s defined in two files %s and %s.' %
-                                               (modname, module_files[modname], s))
-                    module_files[modname] = s
+            filename = os.path.join(self.environment.get_source_dir(),
+                                    s.subdir, s.fname)
+            with open(filename) as f:
+                for line in f:
+                    modmatch = modre.match(line)
+                    if modmatch is not None:
+                        modname = modmatch.group(1)
+                        if modname.lower() == 'procedure':
+                            # MODULE PROCEDURE construct
+                            continue
+                        if modname in module_files:
+                            raise InvalidArguments(
+                                'Namespace collision: module %s defined in '
+                                'two files %s and %s.' %
+                                    (modname, module_files[modname], s))
+                        module_files[modname] = s
         self.fortran_deps[target.get_basename()] = module_files
 
     def get_fortran_deps(self, compiler, src, target):
@@ -1403,27 +1441,32 @@ rule FORTRAN_DEP_HACK
         usere = re.compile(r"\s*use\s+(\w+)", re.IGNORECASE)
         dirname = self.get_target_private_dir(target)
         tdeps= self.fortran_deps[target.get_basename()]
-        for line in open(src):
-            usematch = usere.match(line)
-            if usematch is not None:
-                usename = usematch.group(1)
-                if usename not in tdeps:
-                    # The module is not provided by any source file. This is due to
-                    # a) missing file/typo/etc
-                    # b) using a module provided by the compiler, such as OpenMP
-                    # There's no easy way to tell which is which (that I know of)
-                    # so just ignore this and go on. Ideally we would print a
-                    # warning message to the user but this is a common occurrance,
-                    # which would lead to lots of distracting noise.
-                    continue
-                mod_source_file = tdeps[usename]
-                # Check if a source uses a module it exports itself.
-                # Potential bug if multiple targets have a file with
-                # the same name.
-                if mod_source_file.fname == os.path.split(src)[1]:
-                    continue
-                mod_name = compiler.module_name_to_filename(usematch.group(1))
-                mod_files.append(os.path.join(dirname, mod_name))
+        with open(src) as f:
+            for line in f:
+                usematch = usere.match(line)
+                if usematch is not None:
+                    usename = usematch.group(1)
+                    if usename not in tdeps:
+                        # The module is not provided by any source file. This
+                        # is due to:
+                        #   a) missing file/typo/etc
+                        #   b) using a module provided by the compiler, such as
+                        #      OpenMP
+                        # There's no easy way to tell which is which (that I
+                        # know of) so just ignore this and go on. Ideally we
+                        # would print a warning message to the user but this is
+                        # a common occurrence, which would lead to lots of
+                        # distracting noise.
+                        continue
+                    mod_source_file = tdeps[usename]
+                    # Check if a source uses a module it exports itself.
+                    # Potential bug if multiple targets have a file with
+                    # the same name.
+                    if mod_source_file.fname == os.path.split(src)[1]:
+                        continue
+                    mod_name = compiler.module_name_to_filename(
+                        usematch.group(1))
+                    mod_files.append(os.path.join(dirname, mod_name))
         return mod_files
 
     def get_cross_stdlib_args(self, target, compiler):
@@ -1488,9 +1531,7 @@ rule FORTRAN_DEP_HACK
             else:
                 raise build.InvalidArguments('Invalid source type.')
             abs_src = os.path.join(self.environment.get_build_dir(), rel_src)
-        if isinstance(src, RawFilename):
-            src_filename = src.fname
-        elif isinstance(src, File):
+        if isinstance(src, (RawFilename, File)):
             src_filename = src.fname
         elif os.path.isabs(src):
             src_filename = os.path.basename(src)
