@@ -17,7 +17,7 @@ from . import environment
 from . import dependencies
 from . import mlog
 import copy, os, re
-from .mesonlib import File, flatten, MesonException
+from .mesonlib import File, flatten, MesonException, stringlistify
 from .environment import for_windows, for_darwin
 
 known_basic_kwargs = {'install' : True,
@@ -27,6 +27,7 @@ known_basic_kwargs = {'install' : True,
                       'cpp_args' : True,
                       'cs_args' : True,
                       'vala_args' : True,
+                      'd_args' : True,
                       'link_args' : True,
                       'link_depends': True,
                       'link_with' : True,
@@ -252,12 +253,17 @@ class BuildTarget():
         for s in objects:
             if hasattr(s, 'held_object'):
                 s = s.held_object
-            if isinstance(s, str):
-                self.objects.append(s)
-            elif isinstance(s, ExtractedObjects):
+            if isinstance(s, (str, ExtractedObjects)):
                 self.objects.append(s)
+            elif isinstance(s, (GeneratedList, CustomTarget)):
+                msg =  'Generated files are not allowed in the \'objects\' kwarg ' + \
+                    'for target {!r}.\nIt is meant only for '.format(self.name) + \
+                    'pre-built object files that are shipped with the\nsource ' + \
+                    'tree. Try adding it in the list of sources.'
+                raise InvalidArguments(msg)
             else:
-                raise InvalidArguments('Bad object in target %s.' % self.name)
+                msg = 'Bad object of type {!r} in target {!r}.'.format(type(s).__name__, self.name)
+                raise InvalidArguments(msg)
 
     def process_sourcelist(self, sources):
         if not isinstance(sources, list):
@@ -271,10 +277,11 @@ class BuildTarget():
                 if not s in added_sources:
                     self.sources.append(s)
                     added_sources[s] = True
-            elif isinstance(s, GeneratedList) or isinstance(s, CustomTarget):
+            elif isinstance(s, (GeneratedList, CustomTarget)):
                 self.generated.append(s)
             else:
-                raise InvalidArguments('Bad source in target %s.' % self.name)
+                msg = 'Bad source of type {!r} in target {!r}.'.format(type(s).__name__, self.name)
+                raise InvalidArguments(msg)
 
     def validate_sources(self):
         if len(self.sources) > 0:
@@ -377,6 +384,8 @@ class BuildTarget():
         if not isinstance(valalist, list):
             valalist = [valalist]
         self.add_compiler_args('vala', valalist)
+        dlist = stringlistify(kwargs.get('d_args', []))
+        self.add_compiler_args('d', dlist)
         self.link_args = kwargs.get('link_args', [])
         if not isinstance(self.link_args, list):
             self.link_args = [self.link_args]
@@ -535,8 +544,7 @@ class BuildTarget():
         for t in target:
             if hasattr(t, 'held_object'):
                 t = t.held_object
-            if not isinstance(t, StaticLibrary) and \
-            not isinstance(t, SharedLibrary):
+            if not isinstance(t, (StaticLibrary, SharedLibrary)):
                 raise InvalidArguments('Link target is not library.')
             if self.is_cross != t.is_cross:
                 raise InvalidArguments('Tried to mix cross built and native libraries in target %s.' % self.name)
@@ -601,9 +609,10 @@ class Generator():
         exe = args[0]
         if hasattr(exe, 'held_object'):
             exe = exe.held_object
-        if not isinstance(exe, Executable) and not isinstance(exe, dependencies.ExternalProgram):
+        if not isinstance(exe, (Executable, dependencies.ExternalProgram)):
             raise InvalidArguments('First generator argument must be an executable.')
         self.exe = exe
+        self.depfile = None
         self.process_kwargs(kwargs)
 
     def __repr__(self):
@@ -625,7 +634,6 @@ class Generator():
             if not isinstance(a, str):
                 raise InvalidArguments('A non-string object in "arguments" keyword argument.')
         self.arglist = args
-
         if 'output' not in kwargs:
             raise InvalidArguments('Generator must have "output" keyword argument.')
         outputs = kwargs['output']
@@ -643,12 +651,26 @@ class Generator():
                 if '@OUTPUT@' in o:
                     raise InvalidArguments('Tried to use @OUTPUT@ in a rule with more than one output.')
         self.outputs = outputs
+        if 'depfile' in kwargs:
+            depfile = kwargs['depfile']
+            if not isinstance(depfile, str):
+                raise InvalidArguments('Depfile must be a string.')
+            if os.path.split(depfile)[1] != depfile:
+                raise InvalidArguments('Depfile must be a plain filename without a subdirectory.')
+            self.depfile = depfile
 
     def get_base_outnames(self, inname):
         plainname = os.path.split(inname)[1]
         basename = plainname.split('.')[0]
         return [x.replace('@BASENAME@', basename).replace('@PLAINNAME@', plainname) for x in self.outputs]
 
+    def get_dep_outname(self, inname):
+        if self.depfile is None:
+            raise InvalidArguments('Tried to get dep name for rule that does not have dependency file defined.')
+        plainname = os.path.split(inname)[1]
+        basename = plainname.split('.')[0]
+        return self.depfile.replace('@BASENAME@', basename).replace('@PLAINNAME@', plainname)
+
     def get_arglist(self):
         return self.arglist
 
@@ -814,12 +836,10 @@ class SharedLibrary(BuildTarget):
         elif for_darwin(is_cross, env):
             prefix = 'lib'
             suffix = 'dylib'
-            if self.soversion:
-                # libfoo.X.dylib
-                self.filename_tpl = '{0.prefix}{0.name}.{0.soversion}.{0.suffix}'
-            else:
-                # libfoo.dylib
-                self.filename_tpl = '{0.prefix}{0.name}.{0.suffix}'
+            # libfoo.dylib
+            self.filename_tpl = '{0.prefix}{0.name}.{0.suffix}'
+            # On OS X, the filename should never have the soversion
+            # See: https://github.com/mesonbuild/meson/pull/680
         else:
             prefix = 'lib'
             suffix = 'so'
@@ -868,6 +888,8 @@ class SharedLibrary(BuildTarget):
                 self.vs_module_defs = File.from_absolute_file(path)
             else:
                 self.vs_module_defs = File.from_source_file(environment.source_dir, self.subdir, path)
+            # link_depends can be an absolute path or relative to self.subdir
+            self.link_depends.append(path)
 
     def check_unknown_kwargs(self, kwargs):
         self.check_unknown_kwargs_int(kwargs, known_shlib_kwargs)
@@ -916,11 +938,13 @@ class CustomTarget:
     known_kwargs = {'input' : True,
                     'output' : True,
                     'command' : True,
+                    'capture' : False,
                     'install' : True,
                     'install_dir' : True,
                     'build_always' : True,
                     'depends' : True,
                     'depend_files' : True,
+                    'depfile' : True,
                     }
 
     def __init__(self, name, subdir, kwargs):
@@ -929,6 +953,7 @@ class CustomTarget:
         self.dependencies = []
         self.extra_depends = []
         self.depend_files = [] # Files that this target depends on but are not on the command line.
+        self.depfile = None
         self.process_kwargs(kwargs)
         self.extra_files = []
         self.install_rpath = ''
@@ -953,7 +978,7 @@ class CustomTarget:
         for c in self.sources:
             if hasattr(c, 'held_object'):
                 c = c.held_object
-            if isinstance(c, BuildTarget) or isinstance(c, CustomTarget) or isinstance(c, GeneratedList):
+            if isinstance(c, (BuildTarget, CustomTarget, GeneratedList)):
                 deps.append(c)
         return deps
 
@@ -971,8 +996,19 @@ class CustomTarget:
                 raise InvalidArguments('Output argument not a string.')
             if '/' in i:
                 raise InvalidArguments('Output must not contain a path segment.')
+        self.capture = kwargs.get('capture', False)
+        if self.capture and len(self.output) != 1:
+            raise InvalidArguments(
+                'Capturing can only output to a single file.')
         if 'command' not in kwargs:
             raise InvalidArguments('Missing keyword argument "command".')
+        if 'depfile' in kwargs:
+            depfile = kwargs['depfile']
+            if not isinstance(depfile, str):
+                raise InvalidArguments('Depfile must be a string.')
+            if os.path.split(depfile)[1] != depfile:
+                raise InvalidArguments('Depfile must be a plain filename without a subdirectory.')
+            self.depfile = depfile
         cmd = kwargs['command']
         if not(isinstance(cmd, list)):
             cmd = [cmd]
@@ -980,13 +1016,13 @@ class CustomTarget:
         for i, c in enumerate(cmd):
             if hasattr(c, 'held_object'):
                 c = c.held_object
-            if isinstance(c, str) or isinstance(c, File):
+            if isinstance(c, (str, File)):
                 final_cmd.append(c)
             elif isinstance(c, dependencies.ExternalProgram):
                 if not c.found():
                     raise InvalidArguments('Tried to use not found external program in a build rule.')
                 final_cmd += c.get_command()
-            elif isinstance(c, BuildTarget) or isinstance(c, CustomTarget):
+            elif isinstance(c, (BuildTarget, CustomTarget)):
                 self.dependencies.append(c)
                 final_cmd.append(c)
             elif isinstance(c, list):
@@ -999,6 +1035,10 @@ class CustomTarget:
             else:
                 raise InvalidArguments('Argument %s in "command" is invalid.' % i)
         self.command = final_cmd
+        if self.capture:
+            for c in self.command:
+                if isinstance(c, str) and '@OUTPUT@' in c:
+                    raise InvalidArguments('@OUTPUT@ is not allowed when capturing output.')
         if 'install' in kwargs:
             self.install = kwargs['install']
             if not isinstance(self.install, bool):
@@ -1020,7 +1060,7 @@ class CustomTarget:
         for ed in extra_deps:
             while hasattr(ed, 'held_object'):
                 ed = ed.held_object
-            if not isinstance(ed, CustomTarget) and not isinstance(ed, BuildTarget):
+            if not isinstance(ed, (CustomTarget, BuildTarget)):
                 raise InvalidArguments('Can only depend on toplevel targets.')
             self.extra_depends.append(ed)
         depend_files = kwargs.get('depend_files', [])
