@@ -23,7 +23,7 @@ from . import coredata
 about. To support a new compiler, add its information below.
 Also add corresponding autodetection code in environment.py."""
 
-header_suffixes = ['h', 'hh', 'hpp', 'hxx', 'H', 'ipp', 'moc', 'vapi']
+header_suffixes = ['h', 'hh', 'hpp', 'hxx', 'H', 'ipp', 'moc', 'vapi', 'di']
 cpp_suffixes = ['cc', 'cpp', 'cxx', 'h', 'hh', 'hpp', 'ipp', 'hxx', 'c++']
 c_suffixes = ['c']
 clike_suffixes = c_suffixes + cpp_suffixes
@@ -108,6 +108,27 @@ rust_buildtype_args = {'plain' : [],
                        'minsize' : [],
                        }
 
+d_gdc_buildtype_args = {'plain' : [],
+                       'debug' : ['-g', '-O0'],
+                       'debugoptimized' : ['-g', '-O'],
+                       'release' : ['-O3', '-frelease'],
+                       'minsize' : [],
+                       }
+
+d_ldc_buildtype_args = {'plain' : [],
+                       'debug' : ['-g', '-O0'],
+                       'debugoptimized' : ['-g', '-O'],
+                       'release' : ['-O3', '-release'],
+                       'minsize' : [],
+                       }
+
+d_dmd_buildtype_args = {'plain' : [],
+                       'debug' : ['-g'],
+                       'debugoptimized' : ['-g', '-O'],
+                       'release' : ['-O', '-release'],
+                       'minsize' : [],
+                       }
+
 mono_buildtype_args = {'plain' : [],
                        'debug' : ['-debug'],
                        'debugoptimized': ['-debug', '-optimize+'],
@@ -142,6 +163,7 @@ base_options = {
                                                        ['none', 'address', 'thread', 'undefined', 'memory'],
                                                        'none'),
                 'b_lundef': coredata.UserBooleanOption('b_lundef', 'Use -Wl,--no-undefined when linking', True),
+                'b_asneeded': coredata.UserBooleanOption('b_asneeded', 'Use -Wl,--as-needed when linking', True),
                 'b_pgo': coredata.UserComboOption('b_pgo', 'Use profile guide optimization',
                                                   ['off', 'generate', 'use'],
                                                   'off'),
@@ -223,6 +245,11 @@ def get_base_link_args(options, linker):
             args.append('-Wl,--no-undefined')
     except KeyError:
         pass
+    try:
+        if options['b_asneeded'].value:
+            args.append('-Wl,--as-needed')
+    except KeyError:
+        pass
     try:
         if options['b_coverage'].value:
             args += linker.get_coverage_link_args()
@@ -511,9 +538,8 @@ class CCompiler(Compiler):
         binname += '.exe'
         # Write binary check source
         binary_name = os.path.join(work_dir, binname)
-        ofile = open(source_name, 'w')
-        ofile.write(code)
-        ofile.close()
+        with open(source_name, 'w') as ofile:
+            ofile.write(code)
         # Compile sanity check
         cmdlist = self.exelist + extra_flags + [source_name] + self.get_output_args(binary_name)
         pc = subprocess.Popen(cmdlist, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=work_dir)
@@ -590,9 +616,8 @@ int main () {{ {1}; }}'''
         suflen = len(self.default_suffix)
         (fd, srcname) = tempfile.mkstemp(suffix='.'+self.default_suffix)
         os.close(fd)
-        ofile = open(srcname, 'w')
-        ofile.write(code)
-        ofile.close()
+        with open(srcname, 'w') as ofile:
+            ofile.write(code)
         # Convert flags to the native type of the selected compiler
         args = self.unix_link_flags_to_native(extra_args)
         # Read c_args/cpp_args/etc from the cross-info file (if needed)
@@ -620,9 +645,8 @@ int main () {{ {1}; }}'''
         os.close(fd)
         (fd, dstname) = tempfile.mkstemp()
         os.close(fd)
-        ofile = open(srcname, 'w')
-        ofile.write(code)
-        ofile.close()
+        with open(srcname, 'w') as ofile:
+            ofile.write(code)
         # Convert flags to the native type of the selected compiler
         args = self.unix_link_flags_to_native(extra_args)
         # Select a CRT if needed since we're linking
@@ -645,9 +669,8 @@ int main () {{ {1}; }}'''
             raise CrossNoRunException('Can not run test applications in this cross environment.')
         (fd, srcname) = tempfile.mkstemp(suffix='.'+self.default_suffix)
         os.close(fd)
-        ofile = open(srcname, 'w')
-        ofile.write(code)
-        ofile.close()
+        with open(srcname, 'w') as ofile:
+            ofile.write(code)
         # Convert flags to the native type of the selected compiler
         args = self.unix_link_flags_to_native(extra_args)
         # Select a CRT if needed since we're linking
@@ -875,16 +898,21 @@ int main(int argc, char **argv) {
         # directly try to link via main().
         return self.links('int main() {{ {0}; }}'.format('__builtin_' + funcname), env, args)
 
-    def has_member(self, typename, membername, prefix, env, extra_args=None):
+    def has_members(self, typename, membernames, prefix, env, extra_args=None):
         if extra_args is None:
             extra_args = []
-        templ = '''%s
-void bar() {
-    %s foo;
-    foo.%s;
-};
+        templ = '''{0}
+void bar() {{
+    {1} {2};
+    {3}
+}};
 '''
-        return self.compiles(templ % (prefix, typename, membername), env, extra_args)
+        # Create code that accesses all members
+        members = ''
+        for m in membernames:
+            members += 'foo.{};\n'.format(m)
+        code = templ.format(prefix, typename, 'foo', members)
+        return self.compiles(code, env, extra_args)
 
     def has_type(self, typename, prefix, env, extra_args):
         templ = '''%s
@@ -965,9 +993,9 @@ class ObjCCompiler(CCompiler):
         extra_flags = self.get_cross_extra_flags(environment, compile=True, link=False)
         if self.is_cross:
             extra_flags += self.get_compile_only_args()
-        ofile = open(source_name, 'w')
-        ofile.write('#import<stdio.h>\nint main(int argc, char **argv) { return 0; }\n')
-        ofile.close()
+        with open(source_name, 'w') as ofile:
+            ofile.write('#import<stdio.h>\n'
+                        'int main(int argc, char **argv) { return 0; }\n')
         pc = subprocess.Popen(self.exelist + extra_flags + [source_name, '-o', binary_name])
         pc.wait()
         if pc.returncode != 0:
@@ -999,9 +1027,10 @@ class ObjCPPCompiler(CPPCompiler):
         extra_flags = self.get_cross_extra_flags(environment, compile=True, link=False)
         if self.is_cross:
             extra_flags += self.get_compile_only_args()
-        ofile = open(source_name, 'w')
-        ofile.write('#import<stdio.h>\nclass MyClass;int main(int argc, char **argv) { return 0; }\n')
-        ofile.close()
+        with open(source_name, 'w') as ofile:
+            ofile.write('#import<stdio.h>\n'
+                        'class MyClass;'
+                        'int main(int argc, char **argv) { return 0; }\n')
         pc = subprocess.Popen(self.exelist + extra_flags + [source_name, '-o', binary_name])
         pc.wait()
         if pc.returncode != 0:
@@ -1101,13 +1130,12 @@ class MonoCompiler(Compiler):
         src = 'sanity.cs'
         obj = 'sanity.exe'
         source_name = os.path.join(work_dir, src)
-        ofile = open(source_name, 'w')
-        ofile.write('''public class Sanity {
+        with open(source_name, 'w') as ofile:
+            ofile.write('''public class Sanity {
     static public void Main () {
     }
 }
 ''')
-        ofile.close()
         pc = subprocess.Popen(self.exelist + [src], cwd=work_dir)
         pc.wait()
         if pc.returncode != 0:
@@ -1213,14 +1241,13 @@ class JavaCompiler(Compiler):
         src = 'SanityCheck.java'
         obj = 'SanityCheck'
         source_name = os.path.join(work_dir, src)
-        ofile = open(source_name, 'w')
-        ofile.write('''class SanityCheck {
+        with open(source_name, 'w') as ofile:
+            ofile.write('''class SanityCheck {
   public static void main(String[] args) {
     int i;
   }
 }
 ''')
-        ofile.close()
         pc = subprocess.Popen(self.exelist + [src], cwd=work_dir)
         pc.wait()
         if pc.returncode != 0:
@@ -1249,7 +1276,7 @@ class ValaCompiler(Compiler):
         return False # Because compiles into C.
 
     def get_exelist(self):
-        return self.exelist
+        return self.exelist[:]
 
     def get_werror_args(self):
         return ['--fatal-warnings']
@@ -1260,11 +1287,10 @@ class ValaCompiler(Compiler):
     def sanity_check(self, work_dir, environment):
         src = 'valatest.vala'
         source_name = os.path.join(work_dir, src)
-        ofile = open(source_name, 'w')
-        ofile.write('''class SanityCheck : Object {
+        with open(source_name, 'w') as ofile:
+            ofile.write('''class SanityCheck : Object {
 }
 ''')
-        ofile.close()
         extra_flags = self.get_cross_extra_flags(environment, compile=True, link=False)
         pc = subprocess.Popen(self.exelist + extra_flags + ['-C', '-c', src], cwd=work_dir)
         pc.wait()
@@ -1293,7 +1319,7 @@ class RustCompiler(Compiler):
         return ' '.join(self.exelist)
 
     def get_exelist(self):
-        return self.exelist
+        return self.exelist[:]
 
     def get_id(self):
         return self.id
@@ -1304,11 +1330,10 @@ class RustCompiler(Compiler):
     def sanity_check(self, work_dir, environment):
         source_name = os.path.join(work_dir, 'sanity.rs')
         output_name = os.path.join(work_dir, 'rusttest')
-        ofile = open(source_name, 'w')
-        ofile.write('''fn main() {
+        with open(source_name, 'w') as ofile:
+            ofile.write('''fn main() {
 }
 ''')
-        ofile.close()
         pc = subprocess.Popen(self.exelist + ['-o', output_name, source_name], cwd=work_dir)
         pc.wait()
         if pc.returncode != 0:
@@ -1337,7 +1362,7 @@ class SwiftCompiler(Compiler):
         return self.id
 
     def get_linker_exelist(self):
-        return self.exelist
+        return self.exelist[:]
 
     def name_string(self):
         return ' '.join(self.exelist)
@@ -1346,7 +1371,7 @@ class SwiftCompiler(Compiler):
         return True
 
     def get_exelist(self):
-        return self.exelist
+        return self.exelist[:]
 
     def get_werror_args(self):
         return ['--fatal-warnings']
@@ -1403,10 +1428,9 @@ class SwiftCompiler(Compiler):
         src = 'swifttest.swift'
         source_name = os.path.join(work_dir, src)
         output_name = os.path.join(work_dir, 'swifttest')
-        ofile = open(source_name, 'w')
-        ofile.write('''1 + 2
+        with open(source_name, 'w') as ofile:
+            ofile.write('''1 + 2
 ''')
-        ofile.close()
         extra_flags = self.get_cross_extra_flags(environment, compile=True, link=True)
         pc = subprocess.Popen(self.exelist + extra_flags + ['-emit-executable', '-o', output_name, src], cwd=work_dir)
         pc.wait()
@@ -1419,6 +1443,268 @@ class SwiftCompiler(Compiler):
         suffix = filename.split('.')[-1]
         return suffix in ('swift')
 
+class DCompiler(Compiler):
+    def __init__(self, exelist, version, is_cross):
+        super().__init__(exelist, version)
+        self.id = 'unknown'
+        self.language = 'd'
+        self.is_cross = is_cross
+
+    def sanity_check(self, work_dir, environment):
+        source_name = os.path.join(work_dir, 'sanity.d')
+        output_name = os.path.join(work_dir, 'dtest')
+        with open(source_name, 'w') as ofile:
+            ofile.write('''void main() {
+}
+''')
+        pc = subprocess.Popen(self.exelist + self.get_output_args(output_name) + [source_name], cwd=work_dir)
+        pc.wait()
+        if pc.returncode != 0:
+            raise EnvironmentException('D compiler %s can not compile programs.' % self.name_string())
+        if subprocess.call(output_name) != 0:
+            raise EnvironmentException('Executables created by D compiler %s are not runnable.' % self.name_string())
+
+    def needs_static_linker(self):
+        return True
+
+    def name_string(self):
+        return ' '.join(self.exelist)
+
+    def get_exelist(self):
+        return self.exelist
+
+    def get_id(self):
+        return self.id
+
+    def get_language(self):
+        return self.language
+
+    def can_compile(self, fname):
+        suffix = fname.split('.')[-1]
+        return suffix in ('d', 'di')
+
+    def get_linker_exelist(self):
+        return self.exelist[:]
+
+    def depfile_for_object(self, objfile):
+        return objfile + '.' + self.get_depfile_suffix()
+
+    def get_depfile_suffix(self):
+        return 'dep'
+
+    def get_pic_args(self):
+        return ['-fPIC']
+
+    def get_std_shared_lib_link_args(self):
+        return ['-shared']
+
+    def get_soname_args(self, shlib_name, path, soversion):
+        return []
+
+    def get_unittest_args(self):
+        return ['-unittest']
+
+    def get_buildtype_linker_args(self, buildtype):
+        return []
+
+    def get_std_exe_link_args(self):
+        return []
+
+    def build_rpath_args(self, build_dir, rpath_paths, install_rpath):
+        # This method is to be used by LDC and DMD.
+        # GDC can deal with the verbatim flags.
+        if len(rpath_paths) == 0 and len(install_rpath) == 0:
+            return []
+        paths = ':'.join([os.path.join(build_dir, p) for p in rpath_paths])
+        if len(paths) < len(install_rpath):
+            padding = 'X'*(len(install_rpath) - len(paths))
+            if len(paths) == 0:
+                paths = padding
+            else:
+                paths = paths + ':' + padding
+        return ['-L-rpath={}'.format(paths)]
+
+    def translate_args_to_nongnu(self, args):
+        dcargs = []
+        # Translate common arguments to flags the LDC/DMD compilers
+        # can understand.
+        # The flags might have been added by pkg-config files,
+        # and are therefore out of the user's control.
+        for arg in args:
+            if arg == '-pthread':
+                continue
+            if arg.startswith('-Wl,'):
+                linkargs = arg[arg.index(',')+1:].split(',')
+                for la in linkargs:
+                    dcargs.append('-L' + la.strip())
+                continue
+            elif arg.startswith('-l'):
+                # translate library link flag
+                dcargs.append('-L' + arg)
+                continue
+            dcargs.append(arg)
+
+        return dcargs
+
+class GnuDCompiler(DCompiler):
+    def __init__(self, exelist, version, is_cross):
+        DCompiler.__init__(self, exelist, version, is_cross)
+        self.id = 'gcc'
+        self.warn_args = {'1': ['-Wall', '-Wdeprecated'],
+                          '2': ['-Wall', '-Wextra', '-Wdeprecated'],
+                          '3': ['-Wall', '-Wextra', '-Wdeprecated', '-Wpedantic']}
+        self.base_options = ['b_colorout', 'b_sanitize']
+
+    def get_colorout_args(self, colortype):
+        if mesonlib.version_compare(self.version, '>=4.9.0'):
+            return gnu_color_args[colortype][:]
+        return []
+
+    def get_dependency_gen_args(self, outtarget, outfile):
+        # FIXME: Passing -fmake-deps results in a file-not-found message.
+        # Investigate why.
+        return []
+
+    def get_output_args(self, target):
+        return ['-o', target]
+
+    def get_compile_only_args(self):
+        return ['-c']
+
+    def get_linker_output_args(self, target):
+        return ['-o', target]
+
+    def get_include_args(self, path, is_system):
+        return ['-I' + path]
+
+    def get_warn_args(self, level):
+        return self.warn_args[level]
+
+    def get_werror_args(self):
+        return ['-Werror']
+
+    def get_linker_search_args(self, dirname):
+        return ['-L'+dirname]
+
+    def get_buildtype_args(self, buildtype):
+        return d_gdc_buildtype_args[buildtype]
+
+    def build_rpath_args(self, build_dir, rpath_paths, install_rpath):
+        return build_unix_rpath_args(build_dir, rpath_paths, install_rpath)
+
+    def get_unittest_args(self):
+        return ['-funittest']
+
+class LLVMDCompiler(DCompiler):
+    def __init__(self, exelist, version, is_cross):
+        DCompiler.__init__(self, exelist, version, is_cross)
+        self.id = 'llvm'
+        self.base_options = ['b_coverage', 'b_colorout']
+
+    def get_colorout_args(self, colortype):
+        if colortype == 'always':
+            return ['-enable-color']
+        return []
+
+    def get_dependency_gen_args(self, outtarget, outfile):
+        # LDC using the -deps flag returns a non-Makefile dependency-info file, which
+        # the backends can not use. So we disable this feature for now.
+        return []
+
+    def get_output_args(self, target):
+        return ['-of', target]
+
+    def get_compile_only_args(self):
+        return ['-c']
+
+    def get_linker_output_args(self, target):
+        return ['-of', target]
+
+    def get_include_args(self, path, is_system):
+        return ['-I' + path]
+
+    def get_warn_args(self, level):
+        if level == '2':
+            return ['-wi']
+        else:
+            return ['-w']
+
+    def get_coverage_args(self):
+        return ['-cov']
+
+    def get_buildtype_args(self, buildtype):
+        return d_ldc_buildtype_args[buildtype]
+
+    def get_pic_args(self):
+        return ['-relocation-model=pic']
+
+    def get_linker_search_args(self, dirname):
+        # -L is recognized as "add this to the search path" by the linker,
+        # while the compiler recognizes it as "pass to linker". So, the first
+        # -L is for the compiler, telling it to pass the second -L to the linker.
+        return ['-L-L'+dirname]
+
+    def unix_link_flags_to_native(self, args):
+        return self.translate_args_to_nongnu(args)
+
+    def unix_compile_flags_to_native(self, args):
+        return self.translate_args_to_nongnu(args)
+
+class DmdDCompiler(DCompiler):
+    def __init__(self, exelist, version, is_cross):
+        DCompiler.__init__(self, exelist, version, is_cross)
+        self.id = 'dmd'
+        self.base_options = ['b_coverage', 'b_colorout']
+
+    def get_colorout_args(self, colortype):
+        if colortype == 'always':
+            return ['-color=on']
+        return []
+
+    def get_dependency_gen_args(self, outtarget, outfile):
+        # LDC using the -deps flag returns a non-Makefile dependency-info file, which
+        # the backends can not use. So we disable this feature for now.
+        return []
+
+    def get_output_args(self, target):
+        return ['-of' + target]
+
+    def get_werror_args(self):
+        return ['-w']
+
+    def get_compile_only_args(self):
+        return ['-c']
+
+    def get_linker_output_args(self, target):
+        return ['-of' + target]
+
+    def get_include_args(self, path, is_system):
+        return ['-I' + path]
+
+    def get_warn_args(self, level):
+        return []
+
+    def get_coverage_args(self):
+        return ['-cov']
+
+    def get_linker_search_args(self, dirname):
+        # -L is recognized as "add this to the search path" by the linker,
+        # while the compiler recognizes it as "pass to linker". So, the first
+        # -L is for the compiler, telling it to pass the second -L to the linker.
+        return ['-L-L'+dirname]
+
+    def get_buildtype_args(self, buildtype):
+        return d_dmd_buildtype_args[buildtype]
+
+    def get_std_shared_lib_link_args(self):
+        return ['-shared', '-defaultlib=libphobos2.so']
+
+    def unix_link_flags_to_native(self, args):
+        return self.translate_args_to_nongnu(args)
+
+    def unix_compile_flags_to_native(self, args):
+        return self.translate_args_to_nongnu(args)
+
 class VisualStudioCCompiler(CCompiler):
     std_warn_args = ['/W3']
     std_opt_args= ['/O2']
@@ -1577,9 +1863,8 @@ class VisualStudioCCompiler(CCompiler):
         code = 'int i;\n'
         (fd, srcname) = tempfile.mkstemp(suffix='.'+self.default_suffix)
         os.close(fd)
-        ofile = open(srcname, 'w')
-        ofile.write(code)
-        ofile.close()
+        with open(srcname, 'w') as ofile:
+            ofile.write(code)
         # Read c_args/cpp_args/etc from the cross-info file (if needed)
         extra_args = self.get_cross_extra_flags(env, compile=True, link=False)
         extra_args += self.get_compile_only_args()
@@ -1661,6 +1946,7 @@ class GnuCCompiler(CCompiler):
                              'b_colorout']
         if self.gcc_type != GCC_OSX:
             self.base_options.append('b_lundef')
+            self.base_options.append('b_asneeded')
 
     def get_colorout_args(self, colortype):
         if mesonlib.version_compare(self.version, '>=4.9.0'):
@@ -1731,6 +2017,7 @@ class GnuObjCCompiler(ObjCCompiler):
         self.base_options = ['b_pch', 'b_lto', 'b_pgo', 'b_sanitize', 'b_coverage']
         if self.gcc_type != GCC_OSX:
             self.base_options.append('b_lundef')
+            self.base_options.append('b_asneeded')
 
     def get_buildtype_args(self, buildtype):
         return gnulike_buildtype_args[buildtype]
@@ -1759,6 +2046,7 @@ class GnuObjCPPCompiler(ObjCPPCompiler):
         self.base_options = ['b_pch', 'b_lto', 'b_pgo', 'b_sanitize', 'b_coverage']
         if self.gcc_type != GCC_OSX:
             self.base_options.append('b_lundef')
+            self.base_options.append('b_asneeded')
 
     def get_buildtype_args(self, buildtype):
         return gnulike_buildtype_args[buildtype]
@@ -1780,6 +2068,7 @@ class ClangObjCCompiler(GnuObjCCompiler):
         self.clang_type = cltype
         if self.clang_type != CLANG_OSX:
             self.base_options.append('b_lundef')
+            self.base_options.append('b_asneeded')
 
 class ClangObjCPPCompiler(GnuObjCPPCompiler):
     def __init__(self, exelist, version, cltype, is_cross, exe_wrapper=None):
@@ -1789,6 +2078,7 @@ class ClangObjCPPCompiler(GnuObjCPPCompiler):
         self.base_options = ['b_pch', 'b_lto', 'b_pgo', 'b_sanitize', 'b_coverage']
         if self.clang_type != CLANG_OSX:
             self.base_options.append('b_lundef')
+            self.base_options.append('b_asneeded')
 
 class ClangCCompiler(CCompiler):
     def __init__(self, exelist, version, clang_type, is_cross, exe_wrapper=None):
@@ -1801,6 +2091,7 @@ class ClangCCompiler(CCompiler):
         self.base_options = ['b_pch', 'b_lto', 'b_pgo', 'b_sanitize', 'b_coverage']
         if self.clang_type != CLANG_OSX:
             self.base_options.append('b_lundef')
+            self.base_options.append('b_asneeded')
 
     def get_buildtype_args(self, buildtype):
         return gnulike_buildtype_args[buildtype]
@@ -1835,6 +2126,9 @@ class ClangCCompiler(CCompiler):
     def get_option_link_args(self, options):
         return []
 
+    def has_argument(self, arg, env):
+        return super().has_argument(['-Werror=unknown-warning-option', arg], env)
+
 class GnuCPPCompiler(CPPCompiler):
     # may need to separate the latter to extra_debug_args or something
     std_debug_args = ['-g']
@@ -1850,12 +2144,18 @@ class GnuCPPCompiler(CPPCompiler):
                              'b_colorout']
         if self.gcc_type != GCC_OSX:
             self.base_options.append('b_lundef')
+            self.base_options.append('b_asneeded')
 
     def get_colorout_args(self, colortype):
         if mesonlib.version_compare(self.version, '>=4.9.0'):
             return gnu_color_args[colortype][:]
         return []
 
+    def get_pic_args(self):
+        if self.gcc_type == GCC_MINGW:
+            return [] # On Window gcc defaults to fpic being always on.
+        return ['-fPIC']
+
     def get_always_args(self):
         return ['-pipe']
 
@@ -1911,6 +2211,7 @@ class ClangCPPCompiler(CPPCompiler):
         self.base_options = ['b_pch', 'b_lto', 'b_pgo', 'b_sanitize', 'b_coverage']
         if self.clang_type != CLANG_OSX:
             self.base_options.append('b_lundef')
+            self.base_options.append('b_asneeded')
 
     def get_buildtype_args(self, buildtype):
         return gnulike_buildtype_args[buildtype]
@@ -1942,6 +2243,9 @@ class ClangCPPCompiler(CPPCompiler):
     def get_option_link_args(self, options):
         return []
 
+    def has_argument(self, arg, env):
+        return super().has_argument(['-Werror=unknown-warning-option', arg], env)
+
 class FortranCompiler(Compiler):
     def __init__(self, exelist, version, is_cross, exe_wrapper=None):
         super().__init__(exelist, version)
@@ -1959,7 +2263,7 @@ class FortranCompiler(Compiler):
         return ' '.join(self.exelist)
 
     def get_exelist(self):
-        return self.exelist
+        return self.exelist[:]
 
     def get_language(self):
         return self.language
@@ -1978,12 +2282,11 @@ class FortranCompiler(Compiler):
     def sanity_check(self, work_dir, environment):
         source_name = os.path.join(work_dir, 'sanitycheckf.f90')
         binary_name = os.path.join(work_dir, 'sanitycheckf')
-        ofile = open(source_name, 'w')
-        ofile.write('''program prog
+        with open(source_name, 'w') as ofile:
+            ofile.write('''program prog
      print *, "Fortran compilation is working."
 end program prog
 ''')
-        ofile.close()
         extra_flags = self.get_cross_extra_flags(environment, compile=True, link=True)
         pc = subprocess.Popen(self.exelist + extra_flags + [source_name, '-o', binary_name])
         pc.wait()
@@ -2226,7 +2529,7 @@ class VisualStudioLinker():
         self.exelist = exelist
 
     def get_exelist(self):
-        return self.exelist
+        return self.exelist[:]
 
     def get_std_link_args(self):
         return []
@@ -2278,7 +2581,7 @@ class ArLinker():
         return []
 
     def get_exelist(self):
-        return self.exelist
+        return self.exelist[:]
 
     def get_std_link_args(self):
         return self.std_args
