@@ -15,7 +15,7 @@
 import pickle, os, uuid
 from .mesonlib import MesonException, default_libdir, default_libexecdir, default_prefix
 
-version = '0.33.0'
+version = '0.34.0'
 backendlist = ['ninja', 'vs2010', 'vs2015', 'xcode']
 
 class UserOption:
@@ -154,7 +154,8 @@ class CoreData():
             raise RuntimeError('Tried to set unknown builtin option %s.' % optname)
 
 def load(filename):
-    obj = pickle.load(open(filename, 'rb'))
+    with open(filename, 'rb') as f:
+        obj = pickle.load(f)
     if not isinstance(obj, CoreData):
         raise RuntimeError('Core data file is corrupted.')
     if obj.version != version:
@@ -165,7 +166,8 @@ def load(filename):
 def save(obj, filename):
     if obj.version != version:
         raise RuntimeError('Fatal version mismatch corruption.')
-    pickle.dump(obj, open(filename, 'wb'))
+    with open(filename, 'wb') as f:
+        pickle.dump(obj, f)
 
 def get_builtin_options():
     return list(builtin_options.keys())
@@ -211,6 +213,11 @@ builtin_options = {
         'datadir'           : [ UserStringOption, 'Data file directory.', 'share' ],
         'mandir'            : [ UserStringOption, 'Manual page directory.', 'share/man' ],
         'localedir'         : [ UserStringOption, 'Locale data directory.', 'share/locale' ],
+    # Sysconfdir is a bit special. It defaults to ${prefix}/etc but nobody
+    # uses that. Instead they always set it manually to /etc. This default
+    # value is thus pointless and not really used but we set it to this
+    # for consistency with other systems.
+        'sysconfdir'        : [ UserStringOption, 'Sysconf data directory.', 'etc' ],
         'werror'            : [ UserBooleanOption, 'Treat warnings as errors.', False ],
         'warning_level'     : [ UserComboOption, 'Compiler warning level to use.', [ '1', '2', '3' ], '1'],
         'layout'            : [ UserComboOption, 'Build directory layout.', ['mirror', 'flat' ], 'mirror' ],
