@@ -154,7 +154,7 @@ class PkgConfigDependency(Dependency):
         out = p.communicate()[0]
         if p.returncode != 0:
             raise DependencyException('Could not generate cargs for %s:\n\n%s' % \
-                                      (name, out.decode(errors='ignore')))
+                                      (self.name, out.decode(errors='ignore')))
         self.cargs = out.decode().split()
 
     def _set_libs(self):
@@ -166,7 +166,7 @@ class PkgConfigDependency(Dependency):
         out = p.communicate()[0]
         if p.returncode != 0:
             raise DependencyException('Could not generate libs for %s:\n\n%s' % \
-                                      (name, out.decode(errors='ignore')))
+                                      (self.name, out.decode(errors='ignore')))
         self.libs = []
         for lib in out.decode().split():
             if lib.endswith(".la"):
@@ -228,10 +228,11 @@ class PkgConfigDependency(Dependency):
         return self.is_found
 
     def extract_field(self, la_file, fieldname):
-        for line in open(la_file):
-            arr = line.strip().split('=')
-            if arr[0] == fieldname:
-                return arr[1][1:-1]
+        with open(la_file) as f:
+            for line in f:
+                arr = line.strip().split('=')
+                if arr[0] == fieldname:
+                    return arr[1][1:-1]
         return None
 
     def extract_dlname_field(self, la_file):
@@ -374,7 +375,8 @@ class ExternalProgram():
         shebang and manually parse it to figure out the interpreter to use
         """
         try:
-            first_line = open(script).readline().strip()
+            with open(script) as f:
+                first_line = f.readline().strip()
             if first_line.startswith('#!'):
                 commands = first_line[2:].split('#')[0].strip().split()
                 if mesonlib.is_windows():
@@ -552,12 +554,13 @@ class BoostDependency(Dependency):
         except FileNotFoundError:
             self.version = None
             return
-        for line in ifile:
-            if line.startswith("#define") and 'BOOST_LIB_VERSION' in line:
-                ver = line.split()[-1]
-                ver = ver[1:-1]
-                self.version = ver.replace('_', '.')
-                return
+        with ifile:
+            for line in ifile:
+                if line.startswith("#define") and 'BOOST_LIB_VERSION' in line:
+                    ver = line.split()[-1]
+                    ver = ver[1:-1]
+                    self.version = ver.replace('_', '.')
+                    return
         self.version = None
 
     def detect_src_modules(self):
@@ -1048,6 +1051,17 @@ class SDL2Dependency(Dependency):
         self.is_found = False
         self.cargs = []
         self.linkargs = []
+        try:
+            pcdep = PkgConfigDependency('sdl2', environment, kwargs)
+            if pcdep.found():
+                self.is_found = True
+                self.cargs = pcdep.get_compile_args()
+                self.linkargs = pcdep.get_link_args()
+                self.version = pcdep.get_version()
+                return
+        except Exception as e:
+            mlog.debug('SDL 2 not found via pkgconfig. Trying next, error was:', str(e))
+            pass
         sdlconf = shutil.which('sdl2-config')
         if sdlconf:
             pc = subprocess.Popen(['sdl2-config', '--cflags'],
@@ -1064,16 +1078,7 @@ class SDL2Dependency(Dependency):
             mlog.log('Dependency', mlog.bold('sdl2'), 'found:', mlog.green('YES'), '(%s)' % sdlconf)
             self.version = '2' # FIXME
             return
-        try:
-            pcdep = PkgConfigDependency('sdl2', kwargs)
-            if pcdep.found():
-                self.is_found = True
-                self.cargs = pcdep.get_compile_args()
-                self.linkargs = pcdep.get_link_args()
-                self.version = pcdep.get_version()
-                return
-        except Exception:
-            pass
+        mlog.debug('Could not find sdl2-config binary, trying next.')
         if mesonlib.is_osx():
             fwdep = ExtraFrameworkDependency('sdl2', kwargs.get('required', True))
             if fwdep.found():
