@@ -13,9 +13,12 @@
 # limitations under the License.
 
 import os, re, subprocess, platform
-from . import coredata, mesonlib
+from . import coredata
+from . import mesonlib
+from . import mlog
 from .compilers import *
 import configparser
+import shutil
 
 build_filename = 'meson.build'
 
@@ -578,6 +581,48 @@ class Environment():
             return RustCompiler(exelist, version)
         raise EnvironmentException('Unknown compiler "' + ' '.join(exelist) + '"')
 
+    def detect_d_compiler(self):
+        exelist = None
+        is_cross = False
+        # Search for a D compiler.
+        # We prefer LDC over GDC unless overridden with the DC
+        # environment variable because LDC has a much more
+        # up to date language version at time (2016).
+        if 'DC' in os.environ:
+            exelist = os.environ['DC'].split()
+        elif self.is_cross_build() and want_cross:
+            exelist = [self.cross_info.config['binaries']['d']]
+            is_cross = True
+        elif shutil.which("ldc2"):
+            exelist = ['ldc2']
+        elif shutil.which("ldc"):
+            exelist = ['ldc']
+        elif shutil.which("gdc"):
+            exelist = ['gdc']
+        elif shutil.which("dmd"):
+            exelist = ['dmd']
+        else:
+            raise EnvironmentException('Could not find any supported D compiler.')
+
+        try:
+            p = subprocess.Popen(exelist + ['--version'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        except OSError:
+            raise EnvironmentException('Could not execute D compiler "%s"' % ' '.join(exelist))
+        (out, _) = p.communicate()
+        out = out.decode(errors='ignore')
+        vmatch = re.search(Environment.version_regex, out)
+        if vmatch:
+            version = vmatch.group(0)
+        else:
+            version = 'unknown version'
+        if 'LLVM D compiler' in out:
+            return LLVMDCompiler(exelist, version, is_cross)
+        elif 'gdc' in out:
+            return GnuDCompiler(exelist, version, is_cross)
+        elif 'Digital Mars' in out:
+            return DmdDCompiler(exelist, version, is_cross)
+        raise EnvironmentException('Unknown compiler "' + ' '.join(exelist) + '"')
+
     def detect_swift_compiler(self):
         exelist = ['swiftc']
         try:
@@ -699,30 +744,48 @@ class Environment():
         return self.coredata.get_builtin_option('datadir')
 
 
-def get_args_from_envvars(lang):
-    if lang == 'c':
-        compile_args = os.environ.get('CFLAGS', '').split()
-        link_args = compile_args + os.environ.get('LDFLAGS', '').split()
-        compile_args += os.environ.get('CPPFLAGS', '').split()
-    elif lang == 'cpp':
-        compile_args = os.environ.get('CXXFLAGS', '').split()
-        link_args = compile_args + os.environ.get('LDFLAGS', '').split()
-        compile_args += os.environ.get('CPPFLAGS', '').split()
-    elif lang == 'objc':
-        compile_args = os.environ.get('OBJCFLAGS', '').split()
-        link_args = compile_args + os.environ.get('LDFLAGS', '').split()
-        compile_args += os.environ.get('CPPFLAGS', '').split()
-    elif lang == 'objcpp':
-        compile_args = os.environ.get('OBJCXXFLAGS', '').split()
-        link_args = compile_args + os.environ.get('LDFLAGS', '').split()
-        compile_args += os.environ.get('CPPFLAGS', '').split()
-    elif lang == 'fortran':
-        compile_args = os.environ.get('FFLAGS', '').split()
-        link_args = compile_args + os.environ.get('LDFLAGS', '').split()
-    else:
-        compile_args = []
-        link_args = []
-    return (compile_args, link_args)
+def get_args_from_envvars(lang, compiler_is_linker):
+    """
+    @lang: Language to fetch environment flags for
+
+    Returns a tuple of (compile_flags, link_flags) for the specified language
+    from the inherited environment
+    """
+    def log_var(var, val):
+        if val:
+            mlog.log('Appending {} from environment: {!r}'.format(var, val))
+
+    if lang not in ('c', 'cpp', 'objc', 'objcpp', 'fortran', 'd'):
+        return ([], [])
+
+    # Compile flags
+    cflags_mapping = {'c': 'CFLAGS', 'cpp': 'CXXFLAGS',
+        'objc': 'OBJCFLAGS', 'objcpp': 'OBJCXXFLAGS',
+        'fortran': 'FFLAGS',
+        'd': 'DFLAGS'}
+    compile_flags = os.environ.get(cflags_mapping[lang], '')
+    log_var(cflags_mapping[lang], compile_flags)
+    compile_flags = compile_flags.split()
+
+    # Link flags (same for all languages)
+    link_flags = os.environ.get('LDFLAGS', '')
+    log_var('LDFLAGS', link_flags)
+    link_flags = link_flags.split()
+    if compiler_is_linker:
+        # When the compiler is used as a wrapper around the linker (such as
+        # with GCC and Clang), the compile flags can be needed while linking
+        # too. This is also what Autotools does. However, we don't want to do
+        # this when the linker is stand-alone such as with MSVC C/C++, etc.
+        link_flags = compile_flags + link_flags
+
+    # Pre-processof rlags (not for fortran)
+    preproc_flags = ''
+    if lang in ('c', 'cpp', 'objc', 'objcpp'):
+        preproc_flags = os.environ.get('CPPFLAGS', '')
+    log_var('CPPFLAGS', preproc_flags)
+    compile_flags += preproc_flags.split()
+
+    return (compile_flags, link_flags)
 
 class CrossBuildInfo():
     def __init__(self, filename):
@@ -736,7 +799,7 @@ class CrossBuildInfo():
             raise mesonlib.MesonException('Cross file is missing "binaries".')
 
     def ok_type(self, i):
-        return isinstance(i, str) or isinstance(i, int) or isinstance(i, bool)
+        return isinstance(i, (str, int, bool))
 
     def parse_datafile(self, filename):
         config = configparser.ConfigParser()
