@@ -95,11 +95,7 @@ def is_32bit():
     return not(sys.maxsize > 2**32)
 
 def is_debianlike():
-    try:
-        open('/etc/debian_version', 'r')
-        return True
-    except FileNotFoundError:
-        return False
+    return os.path.isfile('/etc/debian_version')
 
 def exe_exists(arglist):
     try:
@@ -114,7 +110,7 @@ def exe_exists(arglist):
 def detect_vcs(source_dir):
     vcs_systems = [
         dict(name = 'git',        cmd = 'git', repo_dir = '.git', get_rev = 'git describe --dirty=+', rev_regex = '(.*)', dep = '.git/logs/HEAD'),
-        dict(name = 'mercurial',  cmd = 'hg',  repo_dir = '.hg',  get_rev = 'hg id -n',               rev_regex = '(.*)', dep = '.hg/dirstate'),
+        dict(name = 'mercurial',  cmd = 'hg',  repo_dir = '.hg',  get_rev = 'hg id -i',               rev_regex = '(.*)', dep = '.hg/dirstate'),
         dict(name = 'subversion', cmd = 'svn', repo_dir = '.svn', get_rev = 'svn info',               rev_regex = 'Revision: (.*)', dep = '.svn/wc.db'),
         dict(name = 'bazaar',     cmd = 'bzr', repo_dir = '.bzr', get_rev = 'bzr revno',              rev_regex = '(.*)', dep = '.bzr'),
     ]
@@ -261,7 +257,11 @@ def do_mesondefine(line, confdata):
 
 
 def do_conf_file(src, dst, confdata):
-    data = open(src).readlines()
+    try:
+        with open(src) as f:
+            data = f.readlines()
+    except Exception:
+        raise MesonException('Could not read input file %s.' % src)
     # Only allow (a-z, A-Z, 0-9, _, -) as valid characters for a define
     # Also allow escaping '@' with '\@'
     regex = re.compile(r'[^\\]?@([-a-zA-Z0-9_]+)@')
@@ -273,7 +273,8 @@ def do_conf_file(src, dst, confdata):
             line = do_replacement(regex, line, confdata)
         result.append(line)
     dst_tmp = dst + '~'
-    open(dst_tmp, 'w').writelines(result)
+    with open(dst_tmp, 'w') as f:
+        f.writelines(result)
     shutil.copymode(src, dst_tmp)
     replace_if_different(dst, dst_tmp)
 
@@ -303,9 +304,10 @@ def replace_if_different(dst, dst_tmp):
     # If contents are identical, don't touch the file to prevent
     # unnecessary rebuilds.
     try:
-        if open(dst, 'r').read() == open(dst_tmp, 'r').read():
-            os.unlink(dst_tmp)
-            return
+        with open(dst, 'r') as f1, open(dst_tmp, 'r') as f2:
+            if f1.read() == f2.read():
+                os.unlink(dst_tmp)
+                return
     except FileNotFoundError:
         pass
     os.replace(dst_tmp, dst)
