@@ -58,6 +58,7 @@ class Lexer:
             ('plus', re.compile(r'\+')),
             ('dash', re.compile(r'-')),
             ('star', re.compile(r'\*')),
+            ('percent', re.compile(r'\%')),
             ('fslash', re.compile(r'/')),
             ('colon', re.compile(r':')),
             ('equal', re.compile(r'==')),
@@ -67,6 +68,7 @@ class Lexer:
             ('lt', re.compile(r'<')),
             ('ge', re.compile(r'>=')),
             ('gt', re.compile(r'>')),
+            ('questionmark', re.compile(r'\?')),
         ]
 
     def lex(self, code):
@@ -282,6 +284,14 @@ class IfNode():
         self.condition = condition
         self.block = block
 
+class TernaryNode():
+    def __init__(self, lineno, colno, condition, trueblock, falseblock):
+        self.lineno = lineno
+        self.colno = colno
+        self.condition = condition
+        self.trueblock = trueblock
+        self.falseblock = falseblock
+
 class ArgumentNode():
     def __init__(self, token):
         self.lineno = token.lineno
@@ -344,6 +354,7 @@ class Parser:
     def __init__(self, code):
         self.stream = Lexer().lex(code)
         self.getsym()
+        self.in_ternary = False
 
     def getsym(self):
         try:
@@ -383,6 +394,16 @@ class Parser:
                 raise ParseException('Assignment target must be an id.',
                                      left.lineno, left.colno)
             return AssignmentNode(left.lineno, left.colno, left.value, value)
+        elif self.accept('questionmark'):
+            if self.in_ternary:
+                raise ParseException('Nested ternary operators are not allowed.',
+                                     left.lineno, left.colno)
+            self.in_ternary = True
+            trueblock = self.e1()
+            self.expect('colon')
+            falseblock = self.e1()
+            self.in_ternary = False
+            return TernaryNode(left.lineno, left.colno, left, trueblock, falseblock)
         return left
 
     def e2(self):
@@ -414,11 +435,17 @@ class Parser:
         return left
 
     def e5sub(self):
-        left = self.e5mul()
+        left = self.e5mod()
         if self.accept('dash'):
             return ArithmeticNode(left.lineno, left.colno, 'sub', left, self.e5sub())
         return left
 
+    def e5mod(self):
+        left = self.e5mul()
+        if self.accept('percent'):
+            return ArithmeticNode(left.lineno, left.colno, 'mod', left, self.e5mod())
+        return left
+
     def e5mul(self):
         left = self.e5div()
         if self.accept('star'):
